/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Stefan Preuss <stefan.preuss@open-xchange.com>
 * 
 */

function AutoComplete(oText, oDiv, nMaxSize, minChars, timeout) {	
	this.oText = oText;
	this.oDiv = oDiv;
	// minium characters a user has to enter before search will be executed
	this.minChars = minChars && minChars >= 2 ? minChars : 2;
	// maximum numbers of results which will be displayed
	this.nMaxSize = nMaxSize || 20;
	// timeout after the search results will be displayed
	this.timeout = timeout || 500;
	this.searchPattern = new Object();
	this.cancelRequest = false;
	
	/*-- private variables --*/
	this.tt;
	this.cache = new AutoCompleteCache();
	this.currSelected = 0;
	this.currEntries = 0;
			
	// attach handlers to the text-box
	oText.AutoComplete = this;
	oText.onkeydown = AutoComplete.prototype.onTextChange;
	oText.onblur = AutoComplete.prototype.onTextBlur;
}

AutoComplete.prototype.onTextBlur = function() {
   this.AutoComplete.cancelRequest=true;
	this.AutoComplete.clear();
	// remove last ending , if exists
	var value = trimStr(this.AutoComplete.oText.value);
	if (value.charAt(value.length-1) == ",") {
	   this.AutoComplete.oText.value = value.substring(0, value.length-1);
	}
}

AutoComplete.prototype.onTextChange = function(evt) {
	var parent = this;
	if (!evt) evt = event;
	a = evt.keyCode;	
   this.AutoComplete.cancelRequest = false;
	switch (a) {
		case 27: //ESC
			parent.AutoComplete.clear();
			stopEvent(evt); 
			return false;
			break;
		case 38:	// UP
			parent.AutoComplete.keyUpDown(false);
			break;
		case 40:	// DOWN
			parent.AutoComplete.keyUpDown(true);
			break;
		case 16: case 17: case 18: case 35: case 36: case 37: case 39: 
		   // ignore: SHIFT / CTRL / ALT / POS1 / END / LEFT / RIGHT;
		   return true;
		   break;
		case 13: case 9: // ENTER
			return parent.AutoComplete.keyEnter(a);
			break;
		default:
			setTimeout(function() {
					var st = getCurrStr(parent.AutoComplete.oText.value, parent.AutoComplete.oText);					
					if (st && trimStr(st).length >= parent.AutoComplete.minChars) {
						parent.AutoComplete.search(trimStr(st));
					} else {
						parent.AutoComplete.clear();
					}
				},50);
			break;
	}	
}

AutoComplete.prototype.onDivMouseDown = function() {
	var st = this.AutoComplete.oText.scrollTop;
	this.AutoComplete.oText.value = replaceCurrStr(this.AutoComplete.oText.value, this.AutoComplete.oText,this.getAttribute("oval"));
	setSelRange(this.AutoComplete.oText, this.AutoComplete.oText.value.length, this.AutoComplete.oText.value.length, st);
}

AutoComplete.prototype.onDivMouseOver = function() {
	this.className = "background-color-PMG-selection-elements";
	this.AutoComplete.oText.currSelected = this.getAttribute("ono");
}

AutoComplete.prototype.onDivMouseOut = function() {
	this.className = "background-color-content";
}

AutoComplete.prototype.keyUpDown = function (up) {
	// top or bottom has been reached, so nothing to do	
	if ((up && this.currSelected >= this.currEntries) || (!up && this.currSelected == 0))
		return;

	// disable old selection
	document.getElementById("ac_" + this.currSelected).className = "background-color-content";
	if (up)
		this.currSelected++;
	else
		this.currSelected--;
	
	// select new row
	var elem = document.getElementById("ac_" + this.currSelected);
	if (elem) {
		elem.className = "background-color-PMG-selection-elements";
		if (this.oDiv.scrollTop + this.oDiv.clientHeight
            < elem.offsetTop + elem.offsetHeight)
        {
            this.oDiv.scrollTop = elem.offsetTop + elem.offsetHeight
                - this.oDiv.clientHeight;
        }
        if (this.oDiv.scrollTop > elem.offsetTop)
            this.oDiv.scrollTop = elem.offsetTop;
	}
}

AutoComplete.prototype.keyEnter = function () {
	var _rb = true;
	var elem = document.getElementById("ac_" + this.currSelected);
	if (elem) {
		var st = this.oText.scrollTop;
		this.oText.value=replaceCurrStr(this.oText.value, this.oText,elem.getAttribute("oval"));
		setSelRange(this.oText, this.oText.value.length, this.oText.value.length, st);		
		_rb = false;
	}
	this.clear();
	return _rb;
}

AutoComplete.prototype.clear = function () {
	removeChildNodes(this.oDiv);
	this.oDiv.style.height = "";
	if (this.oDiv.style.visibility != "hidden") toggleSelectFields("visible");
	this.oDiv.style.visibility = "hidden";	
	this.currSelected = 0;
	this.currEntries = 0;
}

AutoComplete.prototype.enableDiv = function() {
	// positioning the div and enable it
	this.oDiv.style.top = (getAbsolutePositionTop(this.oText) + this.oText.offsetHeight) + "px";
	this.oDiv.style.left = getAbsolutePositionLeft(this.oText) + "px";		
	this.oDiv.scrollTop = "0";
	this.oDiv.style.visibility = "visible";
	toggleSelectFields("hidden");
}

AutoComplete.prototype.onchange = function(txt) {	
	// get all the matching strings from the AutoCompleteCache
	var aStr = new Array();
	this.cache.getResults(txt, aStr);
	this.currSelected = 0;

	// count the number of strings that match the text-box value
	var nCount = aStr.length > this.nMaxSize ? this.nMaxSize : aStr.length;
	this.currEntries = nCount-1;
			
	// if a suitable number then show the popup-div
	if (!this.cancelRequest && this.nMaxSize != -1 && nCount > 0 && !(nCount == 1 && trimStr(txt) == aStr[0])) {		
		// clear the popup-div.
		while ( this.oDiv.hasChildNodes() )
			this.oDiv.removeChild(this.oDiv.firstChild);

		// add each string to the popup-div
		for (var  i = 0; i < nCount; i++ ) {
			var nDiv = newnode("div", 
				{ overflow: "hidden", whiteSpace: "nowrap", padding: "1px" }, null);
			getParsedNode(txt, aStr[i], nDiv);
			nDiv.id = "ac_" + i;
			nDiv.setAttribute("oval", aStr[i]);
			nDiv.setAttribute("ono", i);
			nDiv.onmousedown = AutoComplete.prototype.onDivMouseDown;
			nDiv.onmouseover = AutoComplete.prototype.onDivMouseOver;
			nDiv.onmouseout = AutoComplete.prototype.onDivMouseOut;
			if (this.currSelected == i)
				nDiv.className = "background-color-PMG-selection-elements";
			nDiv.AutoComplete = this;
			this.oDiv.appendChild(nDiv);			
		}
		//this.oDiv.style.height = "20em";
		if (this.oDiv.offsetHeight > 160)
			this.oDiv.style.height = "160px";
		this.enableDiv();

	} else {
		this.clear();
	}
}

AutoComplete.prototype.search = function(str) {
	var parent = this;		
	
	// remove any quoted strings
	if (str.match(/\"/g))
		str.replace(/\"/g, "");
	
	// already in cache?
	if (this.cache.contains(trimStr(str)))
	   return;
	
	for (var e in this.searchPattern) {
		var sp = new RegExp("^" + e);
		if (sp && str.match(sp)) {
			this.onchange(str);
			return;
		}
	}	
		
	// clear timer if already set
	if (this.tt) 
		window.clearTimeout(this.tt);
		
	if (this.oDiv.firstChild) 
		this.oDiv.removeChild(this.oDiv.firstChild);

	this.oDiv.appendChild(
		newnode("div", { textAlign: "center", width: "100%", padding: "1px" }, null, [ 
			newnode("img",null,{src: getFullImgSrc("img/ox_animated_withoutbg.gif")},[])
		])
	);		
		
	// set new timer with given timout
	this.tt = window.setTimeout(function() {	   
	   if (parent.cancelRequest) return; // request aborted, so leave	
		parent.enableDiv();
		
		// fire-up search
        var pattern = "*" + str + "*";
		var dao = { display_name: pattern, email1: pattern, 
			email2: pattern, email3: pattern, last_name: pattern,
            first_name: pattern, emailAutoComplete: true };
		new JSON().put(AjaxRoot + "/contacts?session=" + session + 
			"&action=search&columns=500,555,556,557",
			dao, null,
			function(cb) {
				if (cb && cb.data) {					
					for (var i = 0; i < cb.data.length; i++) {
						var pn = cb.data[i][0] || ""; // address prefix		
						var em1 = cb.data[i][1]; // email1
						if (em1) parent.cache.add(quotePersonalAddr(pn) + " <" + em1 + ">");
						var em2 = cb.data[i][2]; // email2
						if (em2)	parent.cache.add(quotePersonalAddr(pn) + " <" + em2 + ">");
						var em3 = cb.data[i][3]; // email3
						if (em3) parent.cache.add(quotePersonalAddr(pn) + " <" + em3 + ">");						
					}
					if (!parent.searchPattern[str]) {
						parent.searchPattern[str] = str;
					}
					parent.onchange(str);
				} else {
				   parent.clear();
				}
			});
		} , this.timeout);
}

function quotePersonalAddr(addr) {	
	if (addr.match(new RegExp("[.,:;<>]"))) {
		return "\"" + addr + "\"";
	} else {
		return addr; 
	}
}

function AutoCompleteCache() {
	this.aStr = new Object();
}

AutoCompleteCache.prototype.add = function(str) {
	if ( !this.aStr[str] ) 
		this.aStr[str] = new AutoCompleteCache();
	this.aStr[str] = str;
}

AutoCompleteCache.prototype.getResults = function(str, outStr) {
	for (var i in this.aStr) {
		var re = new RegExp(str, "i");
		if (i.match(re)) 
			outStr.push(i);
	}
}

AutoCompleteCache.prototype.contains = function(str) {
	var tA = new Array();
	this.getResults(str, tA);
	return (tA && tA.length == 1 && tA[0] == str);
}

/******************* UTILS **********************/

/**
 * Constructs a single row with and mark matching text colored
 * @param str  The search String
 * @param oStr The original String (the address in this case)
 * @param oDiv The original div where the row will be appended
 */
function getParsedNode (str, oStr, oDiv) {
	var regexp = new RegExp(str, "gi");	
	var mpos, lIndex = 0;
	while (str != "" && (mpos = regexp.exec(oStr))) {		
		// everything before
	   oDiv.appendChild(document.createTextNode(oStr.substring(lIndex,mpos.index)));
	   lIndex = mpos.index + str.length;
	  	// match
	   oDiv.appendChild(newnode("span", null, { className: "font-style-headline" }, 
	   	[ document.createTextNode(oStr.substring(mpos.index, lIndex)) ] ));
	}
	// everything else
	oDiv.appendChild(document.createTextNode(oStr.substring(lIndex, oStr.length)));
}

/**
 * Gets the current String within the carret position
 * @param str  The search String
 * @param obj The div
 */
function getCurrStr(str, obj) {
	var caretStart = getCaretStart(obj);
	var iLength = 0;
	var io=splitMails(str);
	for (var a = 0; a < io.length; a++) {
		iLength += io[a].length + 1;
		if (iLength > caretStart) {
			return io[a];
		}
	}
	return null;
}

function replaceCurrStr(oldstr,obj,newstr) {	
	var caretStart = getCaretStart(obj);
	var io = splitMails(oldstr);
	var iLength = 0;
	var stringnotfound=true;
	for (var a = 0; a < io.length; a++) {
		iLength += io[a].length + 1;
		if (iLength > caretStart && stringnotfound) {
			io[a]=newstr;
			stringnotfound=false;
		}
	}
	// rebuild string
	var nStr = "";
	for (var a = 0; a < io.length; a++) {
		if (trimStr(io[a]).length != 0) nStr += trimStr(io[a]) + ", ";
	}
 	return nStr;
}

function getCaretStart(obj){
	if (typeof obj.selectionStart != "undefined") {
		return obj.selectionStart;
	} else if (document.selection && document.selection.createRange){
		var O = document.selection.createRange();
		try {
			var am = O.duplicate();
			am.moveToElementText(obj);
		} catch(e) {
			var am = obj.createTextRange();
		}
		am.setEndPoint("EndToStart", O);
		var rb=am.text.length;
		if (rb > obj.value.length) {
			return -1;
		}
		return rb;
	}
}

function setSelRange(obj, selStart, selEnd, sc) {
   if (obj.setSelectionRange) {
      obj.focus(); 
      obj.setSelectionRange(selStart, selEnd);
   } else if (obj.createTextRange) { 
      var range = obj.createTextRange(); 
      range.collapse(true); 
      range.moveEnd('character', selEnd); 
      range.moveStart('character', selStart); 
      range.select(); 
   }   
   setTimeout(function() { 
			obj.focus(); 
			obj.scrollTop = sc + 2 + pxPerEm;		   
   	}, 0);
}

function splitMails(str) {
	var quoted=str.split("\"");
	var noquotes=new Array();
	var holes=new Array();
	var removequotes="";
	var count=0;
	if(quoted.length) {
		for(var i=0;i<quoted.length;i++) {
			if(i%2==0) {
				noquotes.push(quoted[i]);
				count=count+quoted[i].length;
			} else {
				holes.push({count : count , size : quoted[i].length+1 , content : quoted[i] })
			}	
		}
		removequotes=noquotes.join("");
	} else {
		removequotes=str;
	}
	var mysplit=removequotes.split(new RegExp(";|,","g"));
	var newquote=new Array();
	var tmpcount=-1;
	for (var i=0; i<mysplit.length;i++) {
		tmpcount++;
		var tmpInt=mysplit[i].length+tmpcount;
		var tmpText=mysplit[i];
		for(var i2=0;i2<holes.length; i2++) {
			if(holes[i2].count>tmpInt)  { break;}
			if(holes[i2].count<tmpcount) { continue;}
			else {
				tmpText=tmpText.substring(0,holes[i2].count-tmpcount)+"\""+holes[i2].content+"\""+tmpText.substring(holes[i2].count-tmpcount,tmpText.length);
			}
		}
		tmpcount=tmpcount+mysplit[i].length;
		mysplit[i]=tmpText;
	}
	return mysplit;
}
